==> lhcsmapi-1.1.15.tar.gz: lhcsmapi-1.1.15/lhcsmapi/post_mortem/PmDbRequest.py <==
import requests
import json
import warnings

from lhcsmapi.Time import Time

class PmDbRequest(object):
    """PmDbRequest class encapsulates PM database queries in order to simplify the query creation as well as browsing of
    PM events for a given period."""

    _requestTypes = ["pmevent, pmdata", "pmdata/signal"]
    _keywordToValueType = {"system": str, "className": str, "source": str, "signal": str,
                           "fromTimestampInNanos": int, "timestampInNanos": int, "durationInNanos": int}

    @staticmethod
    def generateRequest(requestType, isHeader=False, **kwargs):
        """
            **Generates a PostMortem request**

            Method generates a HTTP REST Post Mortem request query for three request types, with/without PM header,
            and for a given timestamp or a timestamp and duration.

            :param requestType: type of PM request (supported: pmdata, pmdata/signal, pmevent)
            :type requestType: string
            :param isHeader: flag indicating whether (if True) or not (if False) PM header should be requested
            :type isHeader: boolean
            :param kwargs: dictionary of (keyword: argument) pairs which parametrize an db API query
            :type kwargs: dictionary
            :return: string with a PM HTTP request

            - Example::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400000000000
                PmDbRequest.generateRequest("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, durationInNanos = eventDurationSeconds)
               >>> http://pm-api-pro/v2/pmdata/within/duration?source=*&system=QPS&className=DQAMCNMB_PMHSU&fromTimestampInNanos=1426201200000000000&durationInNanos=86400000000000

            - Expected Fail Response::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400000000000
                PmDbRequest.generateRequest("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, duration = eventDurationSeconds)
                >>> ValueError: Some keywords: {'duration'} not in dict_keys(['system', 'className', 'source', 'signal', 'fromTimestampInNanos', 'timestampInNanos', 'durationInNanos'])!

        """

        # check if keys are present
        keyDifference = kwargs.keys() - PmDbRequest._keywordToValueType.keys()
        if len(keyDifference) > 0:
            raise ValueError("Some keywords: {} not in {}!".format(keyDifference, PmDbRequest._keywordToValueType.keys()))

        # check if value types are correct

        # for time setting check if it is either timestampInNanos and not (fromTimestampInNanos or durationInNanos)
        if "timestampInNanos" in kwargs and any(type in ["fromTimestampInNanos", "durationInNanos"] for type in kwargs):
            ValueError(
                "If timestampInNanos is present, then both  fromTimestampInNanos and durationInNanos have to be absent and vice versa!".format(
                    requestType))

        # concatenate keys and values for a query
        keywordsAndArgumentConcat = '&'.join(['%s=%s' % (key, value) for (key, value) in kwargs.items()])

        if not any(type in requestType for type in requestType):
            ValueError("PM request type {} not supported.".format(requestType))

        request = "http://pm-api-pro/v2/{}".format(requestType)

        if isHeader:
            request = request + "/header"

        if "durationInNanos" in kwargs:
            return "{}/within/duration?{}".format(request, keywordsAndArgumentConcat)
        else:
            return "{}?{}".format(request, keywordsAndArgumentConcat)

    @staticmethod
    def getResponse(requestType, isHeader=False, **kwargs):
        """
            **Generates a PM request**

            Method returns content of an REST HTTP Post Mortem request query for three request types, with/without PM header,
            and for a given timestamp or a timestamp and duration.

            :param requestType: type of PM request (supported: pmdata, pmdata/signal, pmevent)
            :type requestType: string
            :param isHeader: flag indicating whether (if True) or not (if False) PM header should be requested
            :type isHeader: boolean
            :param kwargs: dictionary of (keyword: argument) pairs which parametrize an db API query
            :type kwargs: dictionary
            :return: string with a PM HTTP request

            - Example::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400000000000
                PmRequest.getResponse("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, durationInNanos = eventDurationSeconds)
               >>> {'content': [{'valueDescriptor': {'attributes': {'QUENCHTIME': {'description': {'name': 'description',
               >>> ...

            - Expected Fail Response::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400000000000
                PmDbRequest.getResponse("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, duration = eventDurationSeconds)
                >>> ValueError: Some keywords: {'duration'} not in dict_keys(['system', 'className', 'source', 'signal', 'fromTimestampInNanos', 'timestampInNanos', 'durationInNanos'])!

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400
                PmDbRequest.getResponse("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, durationInNanos = eventDurationSeconds)
                >>> UserWarning: Querying Post Mortem failed using the following query:
                                http://pm-api-pro/v2/pmdata/within/duration?source=*&system=QPS&className=DQAMCNMB_PMHSU&fromTimestampInNanos=1426201200000000001&durationInNanos=86400 returned no data so an empty string is being returned

        """
        request = PmDbRequest.generateRequest(requestType, isHeader, **kwargs)
        response = requests.get(request)

        try:
            return json.loads(response.text)
        except:
            warning = """Querying Post Mortem failed using the following query:
                                {} returned no data so an empty string is being returned""".format(request)
            warnings.warn(warning)
            return ''

    @staticmethod
    def findEvents(source, system, className, startTime, endTime=None, duration=None):
        """
            **Finds PM events for a given source, system, className as well as start and end time**

            Method returns content of an REST HTTP Post Mortem request query for three request types, with/without PM header,
            and for a given timestamp or a timestamp and duration. Three operation modes are possible:

            - if endTimeData is not None:

            (startTime, endTime) = (startTimeData, endTimeData)

            - elif len(durationData) is 1:

            (startTime, endTime) = (startTimeData, startTimeData+durationData[0])

            - elif len(durationData) is 2:

            (startTime, endTime) = (startTimeData-durationData[0], startTimeData+durationData[1])

            :param source: PM event source
            :type source: string
            :param system: PM event system
            :type system: string
            :param className: PM event class name
            :type className: string
            :param startTime: input date and time
            :type startTime: int unix timestamp
            :param endTime: unit of the input (None if duration defined)
            :type endTime: int unix timestamp
            :param duration: time zone (None if endTime defined)
            :type duration: list of (time, unit) tuples
            :return: list of (source, timestamp) pairs

            - Example::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 24*60*60

                PmDbRequest.findEvents(source, system, className,
                    eventTime = selectedDayTimeStamp, duration = [(eventDurationSeconds, 's')])
                >>> [('B20L5', 1426220469491000000),
                >>> ...

            - Expected Fail Response::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 1
                PmDbRequest.findEvents(source, system, className,
                    eventTime = selectedDayTimeStamp, duration = [(eventDurationSeconds, 's')])
                >>> UserWarning: Querying Post Mortem failed using the following query:
                                http://pm-api-pro/v2/pmdata/header/within/duration?system=QPS&className=DQAMCNMB_PMHSU&source=*&fromTimestampInNanos=1426201200000000000&durationInNanos=1000000000 returned no data so an empty string is being returned

        """

        fromTimestampInNanos, endTimeInNanos = Time.getQueryPeriodInUnixTime(startTime, endTime, duration)
        MAX_DURATION = int(24 * 60 * 60 * 1e9)
        durationInNanos = endTimeInNanos - fromTimestampInNanos
        durationInNanos = min(durationInNanos, MAX_DURATION)

        # Fetch PM event data
        pmHeaders = PmDbRequest.getResponse("pmdata", True, system = system, className = className, source = source,
                                            fromTimestampInNanos = fromTimestampInNanos, durationInNanos = durationInNanos)

        eventList = []
        for pmHeader in pmHeaders:
            if 'DT' not in pmHeader['source']:
                eventList.append((pmHeader['source'], pmHeader['timestamp']))

            # if 'DT' in 'source', then rubbish data injected into official PM data

        return eventList

==> lhcsmapi-1.1.123.tar.gz: lhcsmapi-1.1.123/lhcsmapi/post_mortem/PmDbRequest.py <==
import requests
import json
import warnings

from lhcsmapi.Time import Time

class PmDbRequest(object):
    """PmDbRequest class encapsulates PM database queries in order to simplify the query creation as well as browsing of
    PM events for a given period."""

    _requestTypes = ["pmevent, pmdata", "pmdata/signal"]
    _keywordToValueType = {"system": str, "className": str, "source": str, "signal": str,
                           "fromTimestampInNanos": int, "timestampInNanos": int, "durationInNanos": int}

    @staticmethod
    def generateRequest(requestType, isHeader=False, pmRestApiPath = "http://pm-api-pro/v2/", **kwargs):
        """
            **Generates a PostMortem request**

            Method generates a HTTP REST Post Mortem request query for three request types, with/without PM header,
            and for a given timestamp or a timestamp and duration.

            :param requestType: type of PM request (supported: pmdata, pmdata/signal, pmevent)
            :type requestType: string
            :param isHeader: flag indicating whether (if True) or not (if False) PM header should be requested
            :type isHeader: boolean
            :param kwargs: dictionary of (keyword: argument) pairs which parametrize an db API query
            :type kwargs: dictionary
            :return: string with a PM HTTP request

            - Example::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400000000000
                PmDbRequest.generateRequest("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, durationInNanos = eventDurationSeconds)
               >>> http://pm-api-pro/v2/pmdata/within/duration?source=*&system=QPS&className=DQAMCNMB_PMHSU&fromTimestampInNanos=1426201200000000000&durationInNanos=86400000000000

            - Expected Fail Response::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400000000000
                PmDbRequest.generateRequest("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, duration = eventDurationSeconds)
                >>> ValueError: Some keywords: {'duration'} not in dict_keys(['system', 'className', 'source', 'signal', 'fromTimestampInNanos', 'timestampInNanos', 'durationInNanos'])!

        """

        # check if keys are present
        keyDifference = kwargs.keys() - PmDbRequest._keywordToValueType.keys()
        if len(keyDifference) > 0:
            raise ValueError("Some keywords: {} not in {}!".format(keyDifference, PmDbRequest._keywordToValueType.keys()))

        # check if value types are correct

        # for time setting check if it is either timestampInNanos and not (fromTimestampInNanos or durationInNanos)
        if "timestampInNanos" in kwargs and any(type in ["fromTimestampInNanos", "durationInNanos"] for type in kwargs):
            ValueError(
                "If timestampInNanos is present, then both  fromTimestampInNanos and durationInNanos have to be absent and vice versa!".format(
                    requestType))

        # concatenate keys and values for a query
        keywordsAndArgumentConcat = '&'.join(['%s=%s' % (key, value) for (key, value) in kwargs.items()])

        if not any(type in requestType for type in requestType):
            ValueError("PM request type {} not supported.".format(requestType))

        request = "{}{}".format(pmRestApiPath, requestType)

        if isHeader:
            request = request + "/header"

        if "durationInNanos" in kwargs:
            return "{}/within/duration?{}".format(request, keywordsAndArgumentConcat)
        else:
            return "{}?{}".format(request, keywordsAndArgumentConcat)

    @staticmethod
    def getResponse(requestType, isHeader=False, displayWarning = False, pmRestApiPath = "http://pm-api-pro/v2/", **kwargs):
        """
            **Generates a PM request**

            Method returns content of an REST HTTP Post Mortem request query for three request types, with/without PM header,
            and for a given timestamp or a timestamp and duration.

            :param requestType: type of PM request (supported: pmdata, pmdata/signal, pmevent)
            :type requestType: string
            :param isHeader: flag indicating whether (if True) or not (if False) PM header should be requested
            :type isHeader: boolean
            :param kwargs: dictionary of (keyword: argument) pairs which parametrize an db API query
            :type kwargs: dictionary
            :return: string with a PM HTTP request

            - Example::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400000000000
                PmRequest.getResponse("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, durationInNanos = eventDurationSeconds)
               >>> {'content': [{'valueDescriptor': {'attributes': {'QUENCHTIME': {'description': {'name': 'description',
               >>> ...

            - Expected Fail Response::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400000000000
                PmDbRequest.getResponse("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, duration = eventDurationSeconds)
                >>> ValueError: Some keywords: {'duration'} not in dict_keys(['system', 'className', 'source', 'signal', 'fromTimestampInNanos', 'timestampInNanos', 'durationInNanos'])!

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 86400
                PmDbRequest.getResponse("pmdata", False, source = source, system = system, className = className,
                    fromTimestampInNanos = selectedDayTimeStamp, durationInNanos = eventDurationSeconds)
                >>> UserWarning: Querying Post Mortem failed using the following query:
                                http://pm-api-pro/v2/pmdata/within/duration?source=*&system=QPS&className=DQAMCNMB_PMHSU&fromTimestampInNanos=1426201200000000001&durationInNanos=86400 returned no data so an empty string is being returned

        """

        request = PmDbRequest.generateRequest(requestType, isHeader, pmRestApiPath, **kwargs)
        response = requests.get(request)

        try:
            return json.loads(response.text)
        except:
            warning = """Querying Post Mortem failed using the following query:
                                            {} returned no data so an empty string is being returned""".format(request)
            if displayWarning:
                warnings.warn(warning)
            return ''

    @staticmethod
    def findEvents(source, system, className, startTime, endTime=None, duration=None, pmRestApiPath = "http://pm-api-pro/v2/", displayWarning = False):
        """
            **Finds PM events for a given source, system, className as well as start and end time**

            Method returns content of an REST HTTP Post Mortem request query for three request types, with/without PM header,
            and for a given timestamp or a timestamp and duration. Three operation modes are possible:

            - if endTimeData is not None:

            (startTime, endTime) = (startTimeData, endTimeData)

            - elif len(durationData) is 1:

            (startTime, endTime) = (startTimeData, startTimeData+durationData[0])

            - elif len(durationData) is 2:

            (startTime, endTime) = (startTimeData-durationData[0], startTimeData+durationData[1])

            :param source: PM event source
            :type source: string
            :param system: PM event system
            :type system: string
            :param className: PM event class name
            :type className: string
            :param startTime: input date and time
            :type startTime: int unix timestamp
            :param endTime: unit of the input (None if duration defined)
            :type endTime: int unix timestamp
            :param duration: time zone (None if endTime defined)
            :type duration: list of (time, unit) tuples
            :return: list of (source, timestamp) pairs

            - Example::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 24*60*60

                PmDbRequest.findEvents(source, system, className,
                    eventTime = selectedDayTimeStamp, duration = [(eventDurationSeconds, 's')])
                >>> [('B20L5', 1426220469491000000),
                >>> ...

            - Expected Fail Response::

                from source.post_mortem.PmDbRequest import PmDbRequest
                source = "*"
                system = "QPS"
                className = "DQAMCNMB_PMHSU"
                selectedDayTimeStamp = 1426201200000000000
                eventDurationSeconds = 1
                PmDbRequest.findEvents(source, system, className,
                    eventTime = selectedDayTimeStamp, duration = [(eventDurationSeconds, 's')])
                >>> UserWarning: Querying Post Mortem failed using the following query:
                                http://pm-api-pro/v2/pmdata/header/within/duration?system=QPS&className=DQAMCNMB_PMHSU&source=*&fromTimestampInNanos=1426201200000000000&durationInNanos=1000000000 returned no data so an empty string is being returned

        """

        fromTimestampInNanos, endTimeInNanos = Time.getQueryPeriodInUnixTime(startTime, endTime, duration)
        MAX_DURATION = int(24 * 60 * 60 * 1e9)
        durationInNanos = endTimeInNanos - fromTimestampInNanos
        durationInNanos = min(durationInNanos, MAX_DURATION)

        # Fetch PM event data
        pmHeaders = PmDbRequest.getResponse("pmdata", True, displayWarning, pmRestApiPath, system=system,
                                            className=className, source=source,
                                            fromTimestampInNanos=fromTimestampInNanos, durationInNanos=durationInNanos)

        if "v2" in pmRestApiPath:
            SOURCE_KEYWORD = "source"
        elif "v3" in pmRestApiPath:
            SOURCE_KEYWORD = "sourceName"

        eventList = []
        for pmHeader in pmHeaders:
            if 'DT' not in pmHeader[SOURCE_KEYWORD]:
                eventList.append((pmHeader[SOURCE_KEYWORD], pmHeader['timestamp']))

            # if 'DT' in 'source', then rubbish data injected into official PM data

        return eventList

    @staticmethod
    def findEventsBetweenDates(startTime, endTime, className, source, system):
        quotient, remainder = Time.moduloDayInUnixTimestamp(startTime, endTime)
        eventDurationNanoSeconds = int(24 * 60 * 60 * 1e9)
        eventSourceAndTimestamp = []
        for day in range(quotient):
            startTimeTemp = Time.convertToUnixTimestamp(startTime) + int(day * eventDurationNanoSeconds)
            eventSourceAndTimestamp += PmDbRequest.findEvents(source, system, className, startTime=startTimeTemp,
                                                              duration=[(eventDurationNanoSeconds, 'ns')])
        if remainder > 0:
            startTimeTemp = Time.convertToUnixTimestamp(endTime) - remainder
            eventSourceAndTimestamp += PmDbRequest.findEvents(source, system, className, startTime=startTimeTemp,
                                                              duration=[(remainder, 'ns')])
        return eventSourceAndTimestamp
